/*
 * GrammarFromXMLParser.h
 *
 *  Created on: Oct 12, 2013
 *      Author: Martin Zak
 */

#ifndef GRAMMAR_FROM_XML_PARSER_H_
#define GRAMMAR_FROM_XML_PARSER_H_

#include "../FromXMLParser.hpp"
#include "Grammar.h"
#include "GrammarFeatures.h"
#include "UnknownGrammar.h"
#include "Unrestricted/UnrestrictedGrammar.h"

#include <list>
#include <set>
#include "../std/variant.hpp"
#include "../sax/Token.h"

namespace grammar {

/**
 * Parser used to get general FSM or EpsilonNFA, NFA, DFA from XML parsed into list of Tokens.
 */
class GrammarFromXMLParser : public alib::FromXMLParser<Grammar, FEATURES> {
protected:
	std::set<alphabet::Symbol> parseNonterminalAlphabet(std::list<sax::Token> &input) const;
	std::set<alphabet::Symbol> parseTerminalAlphabet(std::list<sax::Token> &input) const;
	alphabet::Symbol parseInitialSymbol(std::list<sax::Token> &input) const;

	std::vector<alphabet::Symbol> parseRuleLHS(std::list<sax::Token>& input) const;
	std::vector<alphabet::Symbol> parseRuleRHS(std::list<sax::Token>& input) const;

	template<class T>
	void parseRules(std::list<sax::Token> &input, T& grammar) const;
	void parseRule(std::list<sax::Token>& input, UnknownGrammar& grammar) const;
	void parseRule(std::list<sax::Token>& input, UnrestrictedGrammar& grammar) const;

	virtual Grammar parse(std::list<sax::Token>& input) const;
	virtual Grammar parse(std::list<sax::Token>& input, const std::set<FEATURES>& features) const;
public:
	virtual bool first(std::list<sax::Token>& input) const;

	UnknownGrammar parseUnknownGrammar(std::list<sax::Token>& input) const;
	UnrestrictedGrammar parseUnrestrictedGrammar(std::list<sax::Token>& input) const;
};

} /* namespace grammar */

#endif /* GRAMMAR_FROM_XML_PARSER_H_ */

