/*
 * GrammarToXMLComposer.cpp
 *
 *  Created on: Nov 11, 2013
 *      Author: Jan Travnicek
 */

#include "GrammarToXMLComposer.h"
#include "../alphabet/BlankSymbol.h"

#include "../ToXMLComposers.h"

namespace grammar {

std::list<sax::Token> GrammarToXMLComposer::compose(const GrammarBase& grammar) const {
	std::list<sax::Token> out;
	grammar.Accept((void*) &out, *this);
	return out;
}

std::list<sax::Token> GrammarToXMLComposer::compose(const UnknownGrammar& grammar) const {
	std::list<sax::Token> out;
	out.push_back(sax::Token("grammar", sax::Token::TokenType::START_ELEMENT));
	
	if (grammar.getNonterminalAlphabet().size() > 0)
		composeNonterminalAlphabet(out, grammar.getNonterminalAlphabet());

	if (grammar.getTerminalAlphabet().size() > 0)
		composeTerminalAlphabet(out, grammar.getTerminalAlphabet());

	if (grammar.hasInitialSymbol())
		composeInitialSymbol(out, grammar.getInitialSymbol());

	if (grammar.getRules().size() > 0)
		composeRules(out, grammar);

	out.push_back(sax::Token("grammar", sax::Token::TokenType::END_ELEMENT));
	return out;
}

std::list<sax::Token> GrammarToXMLComposer::compose(const LeftLG& grammar) const {
	std::list<sax::Token> out;

	return out;
}

std::list<sax::Token> GrammarToXMLComposer::compose(const LeftRG& grammar) const {
	std::list<sax::Token> out;

	return out;
}

std::list<sax::Token> GrammarToXMLComposer::compose(const RightLG& grammar) const {
	std::list<sax::Token> out;

	return out;
}

std::list<sax::Token> GrammarToXMLComposer::compose(const RightRG& grammar) const {
	std::list<sax::Token> out;
	return out;
}

std::list<sax::Token> GrammarToXMLComposer::compose(const LG& grammar) const {
	std::list<sax::Token> out;
	return out;
}

std::list<sax::Token> GrammarToXMLComposer::compose(const CFG& grammar) const {
	std::list<sax::Token> out;
	return out;
}

std::list<sax::Token> GrammarToXMLComposer::compose(const EpsilonFreeCFG& grammar) const {
	std::list<sax::Token> out;
	return out;
}

std::list<sax::Token> GrammarToXMLComposer::compose(const CNF& grammar) const {
	std::list<sax::Token> out;
	return out;
}

std::list<sax::Token> GrammarToXMLComposer::compose(const GNF& grammar) const {
	std::list<sax::Token> out;
	return out;
}

std::list<sax::Token> GrammarToXMLComposer::compose(const CSG& grammar) const {
	std::list<sax::Token> out;
	return out;
}

std::list<sax::Token> GrammarToXMLComposer::compose(const NonContractingGrammar& grammar) const {
	std::list<sax::Token> out;
	return out;
}

std::list<sax::Token> GrammarToXMLComposer::compose(const ContextPreservingUnrestrictedGrammar& grammar) const {
	std::list<sax::Token> out;
	return out;
}

std::list<sax::Token> GrammarToXMLComposer::compose(const UnrestrictedGrammar& grammar) const {
	std::list<sax::Token> out;
	out.push_back(sax::Token("UnrestrictedGrammar", sax::Token::TokenType::START_ELEMENT));
	
	composeNonterminalAlphabet(out, grammar.getNonterminalAlphabet());
	composeTerminalAlphabet(out, grammar.getTerminalAlphabet());
	composeInitialSymbol(out, grammar.getInitialSymbol());
	composeRules(out, grammar);

	out.push_back(sax::Token("UnrestrictedGrammar", sax::Token::TokenType::END_ELEMENT));
	return out;
}

void GrammarToXMLComposer::composeNonterminalAlphabet(std::list<sax::Token>& out, const std::set<alphabet::Symbol>& symbols) const {
	out.push_back(sax::Token("nonterminalAlphabet", sax::Token::TokenType::START_ELEMENT));
	for (const auto& symbol : symbols) {
		out.splice(out.end(), alib::ToXMLComposers::symbolComposer.compose(symbol));
	}
	out.push_back(sax::Token("nonterminalAlphabet", sax::Token::TokenType::END_ELEMENT));
}

void GrammarToXMLComposer::composeTerminalAlphabet(std::list<sax::Token>& out, const std::set<alphabet::Symbol>& symbols) const {
	out.push_back(sax::Token("terminalAlphabet", sax::Token::TokenType::START_ELEMENT));
	for (const auto& symbol : symbols) {
		out.splice(out.end(), alib::ToXMLComposers::symbolComposer.compose(symbol));
	}
	out.push_back(sax::Token("terminalAlphabet", sax::Token::TokenType::END_ELEMENT));
}

void GrammarToXMLComposer::composeInitialSymbol(std::list<sax::Token>& out, const alphabet::Symbol& symbol) const {
	out.push_back(sax::Token("initialSymbol", sax::Token::TokenType::START_ELEMENT));
	out.splice(out.end(), alib::ToXMLComposers::symbolComposer.compose(symbol));
	out.push_back(sax::Token("initialSymbol", sax::Token::TokenType::END_ELEMENT));
}


void GrammarToXMLComposer::composeRules(std::list<sax::Token>& out, const UnknownGrammar& grammar) const {
	out.push_back(sax::Token("rules", sax::Token::TokenType::START_ELEMENT));
	for (const auto& rule : grammar.getRules()) {
		out.push_back(sax::Token("rule", sax::Token::TokenType::START_ELEMENT));

		if (rule.getLeftHandSide().size() > 0)
			composeRuleLHS(out, rule.getLeftHandSide());

		if (rule.getLeftHandSide().size() > 0)
			composeRuleRHS(out, rule.getRightHandSide());

		out.push_back(sax::Token("rule", sax::Token::TokenType::END_ELEMENT));
	}

	out.push_back(sax::Token("rules", sax::Token::TokenType::END_ELEMENT));
}

void GrammarToXMLComposer::composeRules(std::list<sax::Token>& out, const UnrestrictedGrammar& grammar) const {
	out.push_back(sax::Token("rules", sax::Token::TokenType::START_ELEMENT));
	for (const auto& rule : grammar.getRules()) {

		for(const auto& rhs : rule.second) {
			out.push_back(sax::Token("rule", sax::Token::TokenType::START_ELEMENT));

			composeRuleLHS(out, rule.first);

			composeRuleRHS(out, rhs);

			out.push_back(sax::Token("rule", sax::Token::TokenType::END_ELEMENT));
		}
	}

	out.push_back(sax::Token("rules", sax::Token::TokenType::END_ELEMENT));
}

void GrammarToXMLComposer::composeRuleLHS(std::list<sax::Token>& out, const std::vector<alphabet::Symbol>& symbols) const {
	out.push_back(sax::Token("lhs", sax::Token::TokenType::START_ELEMENT));
	for (const auto& symbol : symbols) {
		out.splice(out.end(), alib::ToXMLComposers::symbolComposer.compose(symbol));
	}
	out.push_back(sax::Token("lhs", sax::Token::TokenType::END_ELEMENT));
}

void GrammarToXMLComposer::composeRuleRHS(std::list<sax::Token>& out, const std::vector<alphabet::Symbol>& symbols) const {
	out.push_back(sax::Token("rhs", sax::Token::TokenType::START_ELEMENT));
	for (const auto& symbol : symbols) {
		out.splice(out.end(), alib::ToXMLComposers::symbolComposer.compose(symbol));
	}
	out.push_back(sax::Token("rhs", sax::Token::TokenType::END_ELEMENT));
}

std::list<sax::Token> GrammarToXMLComposer::compose(const Grammar& grammar) const {
	std::list<sax::Token> out;
	grammar.getGrammar().Accept((void*) &out, *this);
	return out;
}

void GrammarToXMLComposer::Visit(void* data, const Grammar& grammar) const {
	*((std::list<sax::Token>*) data) = this->compose(grammar);
}

void GrammarToXMLComposer::Visit(void* data, const UnknownGrammar& grammar) const {
	*((std::list<sax::Token>*) data) = this->compose(grammar);
}

void GrammarToXMLComposer::Visit(void* data, const LeftLG& grammar) const {
	*((std::list<sax::Token>*) data) = this->compose(grammar);
}

void GrammarToXMLComposer::Visit(void* data, const LeftRG& grammar) const {
	*((std::list<sax::Token>*) data) = this->compose(grammar);
}

void GrammarToXMLComposer::Visit(void* data, const RightLG& grammar) const {
	*((std::list<sax::Token>*) data) = this->compose(grammar);
}

void GrammarToXMLComposer::Visit(void* data, const RightRG& grammar) const {
	*((std::list<sax::Token>*) data) = this->compose(grammar);
}

void GrammarToXMLComposer::Visit(void* data, const LG& grammar) const {
	*((std::list<sax::Token>*) data) = this->compose(grammar);
}

void GrammarToXMLComposer::Visit(void* data, const CFG& grammar) const {
	*((std::list<sax::Token>*) data) = this->compose(grammar);
}

void GrammarToXMLComposer::Visit(void* data, const EpsilonFreeCFG& grammar) const {
	*((std::list<sax::Token>*) data) = this->compose(grammar);
}

void GrammarToXMLComposer::Visit(void* data, const CNF& grammar) const {
	*((std::list<sax::Token>*) data) = this->compose(grammar);
}

void GrammarToXMLComposer::Visit(void* data, const GNF& grammar) const {
	*((std::list<sax::Token>*) data) = this->compose(grammar);
}

void GrammarToXMLComposer::Visit(void* data, const CSG& grammar) const {
	*((std::list<sax::Token>*) data) = this->compose(grammar);
}

void GrammarToXMLComposer::Visit(void* data, const NonContractingGrammar& grammar) const {
	*((std::list<sax::Token>*) data) = this->compose(grammar);
}

void GrammarToXMLComposer::Visit(void* data, const ContextPreservingUnrestrictedGrammar& grammar) const {
	*((std::list<sax::Token>*) data) = this->compose(grammar);
}

void GrammarToXMLComposer::Visit(void* data, const UnrestrictedGrammar& grammar) const {
	*((std::list<sax::Token>*) data) = this->compose(grammar);
}

} /* namespace grammar */

