/*
 * GrammarToXMLComposer.h
 *
 *  Created on: Nov 11, 2013
 *      Author: Jan Travnicek
 */

#ifndef GRAMMAR_TO_XML_COMPOSER_H_
#define GRAMMAR_TO_XML_COMPOSER_H_

#include <string>
#include <list>
#include "Grammar.h"
#include "UnknownGrammar.h"
class LeftLG;
class LeftRG;
class RightLG;
class RightRG;
class LG;
class CFG;
class EpsilonFreeCFG;
class CNF;
class GNF;
class CSG;
class NonContractingGrammar;
#include "Unrestricted/ContextPreservingUnrestrictedGrammar.h"
#include "Unrestricted/UnrestrictedGrammar.h"
#include "../sax/Token.h"

namespace grammar {

/**
 * This class contains methods to print XML representation of automata to the output stream.
 */
class GrammarToXMLComposer : public GrammarBase::const_visitor_type {
	void Visit(void*, const Grammar& grammar) const;
	void Visit(void*, const UnknownGrammar& grammar) const;
	void Visit(void*, const LeftLG& grammar) const;
	void Visit(void*, const LeftRG& grammar) const;
	void Visit(void*, const RightLG& grammar) const;
	void Visit(void*, const RightRG& grammar) const;
	void Visit(void*, const LG& grammar) const;
	void Visit(void*, const CFG& grammar) const;
	void Visit(void*, const EpsilonFreeCFG& grammar) const;
	void Visit(void*, const CNF& grammar) const;
	void Visit(void*, const GNF& grammar) const;
	void Visit(void*, const CSG& grammar) const;
	void Visit(void*, const NonContractingGrammar& grammar) const;
	void Visit(void*, const ContextPreservingUnrestrictedGrammar& grammar) const;
	void Visit(void*, const UnrestrictedGrammar& grammar) const;

	void composeNonterminalAlphabet(std::list<sax::Token>& out, const std::set<alphabet::Symbol>& symbols) const;
	void composeTerminalAlphabet(std::list<sax::Token>& out, const std::set<alphabet::Symbol>& symbols) const;
	void composeInitialSymbol(std::list<sax::Token>& out, const alphabet::Symbol& symbol) const;
	
	void composeRules(std::list<sax::Token>& out, const UnknownGrammar& grammar) const;
	void composeRules(std::list<sax::Token>& out, const UnrestrictedGrammar& grammar) const;

	void composeRuleLHS(std::list<sax::Token>& out, const std::vector<alphabet::Symbol>& symbols) const;
	void composeRuleRHS(std::list<sax::Token>& out, const std::vector<alphabet::Symbol>& symbols) const;
public:
	/**
	 * Prints XML representation of UnknownGrammar to the output stream.
	 * @param grammar grammar to print
	 * @return list of xml tokens representing the grammar
	 */
	std::list<sax::Token> compose(const GrammarBase& grammar) const;

	std::list<sax::Token> compose(const Grammar& grammar) const;

	std::list<sax::Token> compose(const UnknownGrammar& grammar) const;
	std::list<sax::Token> compose(const LeftLG& grammar) const;
	std::list<sax::Token> compose(const LeftRG& grammar) const;
	std::list<sax::Token> compose(const RightLG& grammar) const;
	std::list<sax::Token> compose(const RightRG& grammar) const;
	std::list<sax::Token> compose(const LG& grammar) const;
	std::list<sax::Token> compose(const CFG& grammar) const;
	std::list<sax::Token> compose(const EpsilonFreeCFG& grammar) const;
	std::list<sax::Token> compose(const CNF& grammar) const;
	std::list<sax::Token> compose(const GNF& grammar) const;
	std::list<sax::Token> compose(const CSG& grammar) const;
	std::list<sax::Token> compose(const NonContractingGrammar& grammar) const;
	std::list<sax::Token> compose(const ContextPreservingUnrestrictedGrammar& grammar) const;
	std::list<sax::Token> compose(const UnrestrictedGrammar& grammar) const;
};

} /* namespace grammar */

#endif /* GRAMMAR_TO_XML_COMPOSER_H_ */

