/*
 * ContextPreservingUnrestrictedGrammar.h
 *
 *  Created on: Nov 17, 2013
 *      Author: Jan Travnicek
 */

#ifndef CONTEXT_PRESERVING_UNRESTRICTED_GRAMMAR_H_
#define CONTEXT_PRESERVING_UNRESTRICTED_GRAMMAR_H_

#include "../GrammarBase.h"
#include <map>
#include "../common/TerminalNonterminalAlphabetInitialSymbol.h"

namespace grammar {

/**
 * UnrestrictedGrammar grammar. Type 0 in Chomsky hierarchy. Produces recursively enumerable language.
 */
class ContextPreservingUnrestrictedGrammar : public std::element<ContextPreservingUnrestrictedGrammar, GrammarBase>, public TerminalNonterminalAlphabetInitialSymbol {
	std::map<std::vector<alphabet::Symbol>, std::set<std::vector<alphabet::Symbol>>> rules;

public:
	ContextPreservingUnrestrictedGrammar(const alphabet::Symbol& initialSymbol);

	ContextPreservingUnrestrictedGrammar(const std::set<alphabet::Symbol>& nonTerminalSymbols, const std::set<alphabet::Symbol>& terminalSymbols, const alphabet::Symbol& initialSymbol);

	virtual GrammarBase* clone() const;

	virtual GrammarBase* plunder() &&;

	bool addRule(const std::vector<alphabet::Symbol>& leftHandSide, const std::vector<alphabet::Symbol>& rightHandSide);

	const std::map<std::vector<alphabet::Symbol>, std::set<std::vector<alphabet::Symbol>>> getRules() const;

	bool removeRule(const std::vector<alphabet::Symbol>& leftHandSide, const std::vector<alphabet::Symbol>& rightHandSide);

	bool removeTerminalSymbol(const alphabet::Symbol& symbol);

	bool removeNonterminalSymbol(const alphabet::Symbol& symbol);

	virtual bool operator==(const GrammarBase& other) const;

	virtual bool operator==(const ContextPreservingUnrestrictedGrammar& other) const;

	virtual void operator>>(std::ostream& os) const;
};

} /* namespace grammar */

#endif /* CONTEXT_PRESERVING_UNRESTRICTED_GRAMMAR_H_ */
