/*
 * TerminalNonterminalAlphabetAlphabetInitialSymbol.cpp
 *
 *  Created on: Nov 2, 2013
 *      Author: Martin Zak
 */

#include "TerminalNonterminalAlphabetInitialSymbol.h"
#include "../GrammarException.h"
#include "../../alphabet/LabeledSymbol.h"
#include "../../label/Label.h"
#include "../../label/NextLabel.h"
#include "../../alphabet/LabeledSymbol.h"
#include "../../alphabet/Symbol.h"
#include <limits.h>

namespace grammar {

TerminalNonterminalAlphabetInitialSymbol::TerminalNonterminalAlphabetInitialSymbol(const alphabet::Symbol& initialSymbol) : initialSymbol(initialSymbol) {
	addNonterminalSymbol(initialSymbol);
}

bool TerminalNonterminalAlphabetInitialSymbol::addTerminalSymbol(const alphabet::Symbol& symbol) {
	if(nonterminalAlphabet.find(symbol) != nonterminalAlphabet.end()){
		throw GrammarException("Symbol \"" + (std::string) symbol.getSymbol() + "\" is nonterminal symbol.");
	}

	return terminalAlphabet.insert(symbol).second;
}

const std::set<alphabet::Symbol>& TerminalNonterminalAlphabetInitialSymbol::getTerminalAlphabet() const {
	return terminalAlphabet;
}

void TerminalNonterminalAlphabetInitialSymbol::setTerminalAlphabet(const std::set<alphabet::Symbol>& alphabet) {
	std::set<alphabet::Symbol> removed;
	std::set_difference(terminalAlphabet.begin(), terminalAlphabet.end(), alphabet.begin(), alphabet.end(), std::inserter(removed, removed.end()));

	std::set<alphabet::Symbol> added;
	std::set_difference(alphabet.begin(), alphabet.end(), terminalAlphabet.begin(), terminalAlphabet.end(), std::inserter(added, added.end()));
	
	for(const alphabet::Symbol& removedSymbol : removed) {
		removeTerminalSymbol(removedSymbol);
	}

	for(const alphabet::Symbol& addedSymbol : added) {
		addTerminalSymbol(addedSymbol);
	}
}

bool TerminalNonterminalAlphabetInitialSymbol::addNonterminalSymbol(const alphabet::Symbol& symbol) {
	if(terminalAlphabet.find(symbol) != terminalAlphabet.end()){
		throw GrammarException("Symbol \"" + (std::string) symbol.getSymbol() + "\" is terminal symbol.");
	}

	return nonterminalAlphabet.insert(symbol).second;
}

const std::set<alphabet::Symbol>& TerminalNonterminalAlphabetInitialSymbol::getNonterminalAlphabet() const {
	return nonterminalAlphabet;
}

void TerminalNonterminalAlphabetInitialSymbol::setNonterminalAlphabet(const std::set<alphabet::Symbol>& alphabet) {
	std::set<alphabet::Symbol> removed;
	std::set_difference(nonterminalAlphabet.begin(), nonterminalAlphabet.end(), alphabet.begin(), alphabet.end(), std::inserter(removed, removed.end()));

	std::set<alphabet::Symbol> added;
	std::set_difference(alphabet.begin(), alphabet.end(), nonterminalAlphabet.begin(), nonterminalAlphabet.end(), std::inserter(added, added.end()));
	
	for(const alphabet::Symbol& removedSymbol : removed) {
		removeNonterminalSymbol(removedSymbol);
	}

	for(const alphabet::Symbol& addedSymbol : added) {
		addNonterminalSymbol(addedSymbol);
	}
}

const alphabet::Symbol& TerminalNonterminalAlphabetInitialSymbol::getInitialSymbol() const{
	return initialSymbol;
}

void TerminalNonterminalAlphabetInitialSymbol::setInitialSymbol(const alphabet::Symbol& symbol) {
	if(nonterminalAlphabet.find(symbol) == nonterminalAlphabet.end()) {
		throw GrammarException("Symbol \"" + (std::string) symbol.getSymbol() + "\" isn't nonterminal symbol.");
	}

	initialSymbol = symbol;
}

alphabet::Symbol TerminalNonterminalAlphabetInitialSymbol::createUniqueSymbol(const alphabet::Symbol& base, const std::set<alphabet::Symbol>& terminalAlphabet, const std::set<alphabet::Symbol>& nonterminalAlphabet) {
	label::NextLabel nextLabelCreator;

	const alphabet::LabeledSymbol* baseSymbol = dynamic_cast<const alphabet::LabeledSymbol*>(&(base.getSymbol()));
	if(baseSymbol == NULL)
		throw GrammarException("Could not create unique symbol with nonlabeled base symbol " + (std::string) base.getSymbol() + "." );

	int i = 0;
	do {
		label::Label nextLabel = nextLabelCreator.nextLabel(baseSymbol->getLabel());
		alphabet::Symbol attempt = alphabet::Symbol(alphabet::LabeledSymbol(nextLabel));
		if(terminalAlphabet.count(attempt) == 0 && nonterminalAlphabet.count(attempt) == 0)
			return attempt;
	} while(++i < INT_MAX);

	throw GrammarException("Could not create unique symbol with base symbol " + (std::string) base.getSymbol() + "." );
}

} /* namespace grammar */
